
#include <windows.h>

struct {
	char* old_bytes;
	char* new_bytes;
}
patch_61[] = {
	// is_on_blacklist
	"8B4C2474 5F 5E 5B 33CC B001 E8........ 83 C4 6C",
	"8B4C2474 5F 5E 5B 33CC B000 E8........ 83 C4 6C",

	// check_original_user
	"5E 32C0 5B 8B8C24A0000000 33CC E8........ 81C4A4000000 C3",
	"5E B001 5B 8B8C24A0000000 33CC E8........ 81C4A4000000 C3",

	// wrong_user_credentials
	"3C20 7308 3C0A 0F85........ 8A4101",
	"3C20 EB08 3C0A 0F85........ 8A4101",
};

class PEFile
{
public:

	PEFile(LPVOID pImageBase) {
		Attach(pImageBase);
	}

	bool Attach(LPVOID pImageBase)
	{
		m_pNTHeader = 0;

		if(!pImageBase)
			return false;

		PIMAGE_DOS_HEADER pMZHeader = (PIMAGE_DOS_HEADER)pImageBase;
		
		if(pMZHeader->e_magic != IMAGE_DOS_SIGNATURE)
			return false;

		PIMAGE_NT_HEADERS pNTHeader = (PIMAGE_NT_HEADERS)((DWORD)pMZHeader + pMZHeader->e_lfanew);

		if(pNTHeader->Signature != IMAGE_NT_SIGNATURE)
			return false;

		m_pNTHeader = pNTHeader;
		return true;
	}

	PIMAGE_SECTION_HEADER SectionsTable() const	{
		return (PIMAGE_SECTION_HEADER)((DWORD)m_pNTHeader + sizeof(IMAGE_NT_HEADERS));
	}

	int GetNumberOfSections() const	{
		return m_pNTHeader->FileHeader.NumberOfSections;
	}

	DWORD GetSectionRVA(int num) const {
		return SectionsTable()[num].VirtualAddress;
	}

	DWORD GetSectionVirtualSize(int num) const {
		return SectionsTable()[num].Misc.VirtualSize;
	}

protected:

	PIMAGE_NT_HEADERS m_pNTHeader;
};

#define SEARCH_NOT_FOUND (-1)

class SearchPattern
{
public:

	SearchPattern() : m_size(0) {}

	void SetPattern(const char* pattern)
	{
		m_size = 0;

		while(*pattern)
		{
			unsigned short w1;
			unsigned short w2;

			if((w1 = (unsigned char)*pattern++) == ' ')
				continue;

			if(!(w2 = (unsigned char)*pattern++))
				break;

			if(w1 >= '0' && w1 <= '9')
				w1 &= ~0x30;
			else
			{
				w1 &= ~0x20;

				if(w1 >= 'A' && w1 <= 'F')
					w1 -= 'A' - 0xa;
				else
				{
					m_pattern[m_size++] = -1;
					continue;
				}
			}

			if(w2 >= '0' && w2 <= '9')
				w2 &= ~0x30;
			else
			{
				w2 &= ~0x20;

				if(w2 >= 'A' && w2 <= 'F')
					w2 -= 'A' - 0xa;
				else
				{
					m_pattern[m_size++] = -1;
					continue;
				}
			}

			m_pattern[m_size++] = w2 | (w1 << 4);
		}
	}

	size_t FindPattern(const unsigned char* ptr,size_t size)
	{
		if(size < m_size)
			return SEARCH_NOT_FOUND;

		for(size_t i = 0; i < size - m_size + 1; ++i)
		{
			const short* pat = m_pattern;
			const short* end = m_pattern + m_size;

			for(const unsigned char* pi = ptr + i ; pat < end; ++pat, ++pi)
			{
				if(*pat >= 0 && *pat != *pi)
					break;
			}

			if(pat == end)
				return i;
		}

		return SEARCH_NOT_FOUND;
	}

	bool CopyPattern(unsigned char* ptr,size_t size)
	{
		if(size < m_size)
			return false;

		for(size_t i = 0; i < m_size; ++i)
		{
			if(m_pattern[i] >= 0)
				ptr[i] = (unsigned char)m_pattern[i];
		}

		return true;
	}

	size_t GetPatternSize() const {
		return m_size;
	}

protected:

	short m_pattern[1024];
	size_t m_size;
};

void MsgBox(const char* text)
{
	::MessageBox(NULL,text,"dbfix",MB_ICONINFORMATION);
}

DWORD Protect(PVOID base,DWORD size,DWORD flags)
{
	DWORD dwTemp = 0;
	::VirtualProtect(base,size,flags,&dwTemp);
	return dwTemp;
}

int StrCmpN(const char* first,const char* last,size_t count)
{
	if(!count)
		return 0;

	while(--count && *first && *first == *last)
	{
		first++;
		last++;
	}

	return((int)(*first - *last));
}

BOOL WINAPI DllMain(HMODULE hDllHandle,DWORD dwReason,LPVOID lpvReserved)
{
	char mutexName[256];

	switch(dwReason)
	{
	case DLL_PROCESS_ATTACH:

		::DisableThreadLibraryCalls(hDllHandle);

		wsprintf(mutexName,"IDAPRO.DATABASE.FIX.%d",GetCurrentProcessId());

		if(::CreateMutex(NULL,FALSE,mutexName) &&
			::GetLastError() != ERROR_ALREADY_EXISTS)
		{
			DWORD base = (DWORD)::GetModuleHandle("ida.wll");

			if(!base)
				base = (DWORD)::GetModuleHandle("ida64.wll");

			if(!base)
			{
				MsgBox("Couldn't find ida.wll nor ida64.wll");
				return FALSE;
			}

 			PEFile pe((PVOID)base);
			DWORD rva = 0, vsize = 0;
 
 			for(int i = 0; i < pe.GetNumberOfSections(); ++i)
 			{
 				if(StrCmpN((char*)pe.SectionsTable()[i].Name,".text",IMAGE_SIZEOF_SHORT_NAME) == 0)
 				{
 					rva = pe.GetSectionRVA(i);
 					vsize = pe.GetSectionVirtualSize(i);
 					break;
 				}
 			}

			if(!rva || !vsize)
			{
				MsgBox("Couldn't find ida.wll '.text' section");
				return FALSE;
			}

			DWORD vaddr = base + rva;

			for(int i = 0; i < _countof(patch_61); ++i)
			{
				SearchPattern spo,spn;
				spo.SetPattern(patch_61[i].old_bytes);
				spn.SetPattern(patch_61[i].new_bytes);

				PBYTE addr = 0;

				size_t n = spo.FindPattern((PBYTE)vaddr,vsize);

				if(n != SEARCH_NOT_FOUND)
				{
					addr = (PBYTE)(vaddr + n);

					n = spo.FindPattern((PBYTE)vaddr + (n + 1),vsize - (n + 1));

					if(n != SEARCH_NOT_FOUND)
						addr = 0;
				}

				if(addr)
				{
					DWORD prot = Protect(addr,spn.GetPatternSize(),PAGE_READWRITE);
					spn.CopyPattern(addr,spn.GetPatternSize());
					Protect(addr,spn.GetPatternSize(),prot);
				}
				else
				{
					MsgBox("Unknown version of Idapro");
					return FALSE;
				}
			}
		}
		break;
	}

	return TRUE;
}
